@@ -2,59 +2,91 @@
  * ncat_connect.c -- Ncat connect mode.                                    *
  ***********************IMPORTANT NMAP LICENSE TERMS************************
  *                                                                         *
- * The Nmap Security Scanner is (C) 1996-2012 Insecure.Com LLC. Nmap is    *
+ * The Nmap Security Scanner is (C) 1996-2013 Insecure.Com LLC. Nmap is    *
  * also a registered trademark of Insecure.Com LLC.  This program is free  *
  * software; you may redistribute and/or modify it under the terms of the  *
  * GNU General Public License as published by the Free Software            *
- * Foundation; Version 2 with the clarifications and exceptions described  *
- * below.  This guarantees your right to use, modify, and redistribute     *
- * this software under certain conditions.  If you wish to embed Nmap      *
- * technology into proprietary software, we sell alternative licenses      *
- * (contact sales@insecure.com).  Dozens of software vendors already       *
- * license Nmap technology such as host discovery, port scanning, OS       *
- * detection, version detection, and the Nmap Scripting Engine.            *
+ * Foundation; Version 2 ("GPL"), BUT ONLY WITH ALL OF THE CLARIFICATIONS  *
+ * AND EXCEPTIONS DESCRIBED HEREIN.  This guarantees your right to use,    *
+ * modify, and redistribute this software under certain conditions.  If    *
+ * you wish to embed Nmap technology into proprietary software, we sell    *
+ * alternative licenses (contact sales@insecure.com).  Dozens of software  *
+ * vendors already license Nmap technology such as host discovery, port    *
+ * scanning, OS detection, version detection, and the Nmap Scripting       *
+ * Engine.                                                                 *
  *                                                                         *
- * Note that the GPL places important restrictions on "derived works", yet *
- * it does not provide a detailed definition of that term.  To avoid       *
+ * Note that the GPL places important restrictions on "derivative works",  *
+ * yet it does not provide a detailed definition of that term.  To avoid   *
  * misunderstandings, we interpret that term as broadly as copyright law   *
  * allows.  For example, we consider an application to constitute a        *
- * "derivative work" for the purpose of this license if it does any of the *
- * following:                                                              *
- * o Integrates source code from Nmap                                      *
- * o Reads or includes Nmap copyrighted data files, such as                *
- *   nmap-os-db or nmap-service-probes.                                    *
- * o Executes Nmap and parses the results (as opposed to typical shell or  *
- *   execution-menu apps, which simply display raw Nmap output and so are  *
- *   not derivative works.)                                                *
- * o Integrates/includes/aggregates Nmap into a proprietary executable     *
- *   installer, such as those produced by InstallShield.                   *
- * o Links to a library or executes a program that does any of the above   *
+ * derivative work for the purpose of this license if it does any of the   *
+ * following with any software or content covered by this license          *
+ * ("Covered Software"):                                                   *
  *                                                                         *
- * The term "Nmap" should be taken to also include any portions or derived *
- * works of Nmap, as well as other software we distribute under this       *
- * license such as Zenmap, Ncat, and Nping.  This list is not exclusive,   *
- * but is meant to clarify our interpretation of derived works with some   *
- * common examples.  Our interpretation applies only to Nmap--we don't     *
- * speak for other people's GPL works.                                     *
+ * o Integrates source code from Covered Software.                         *
  *                                                                         *
- * If you have any questions about the GPL licensing restrictions on using *
- * Nmap in non-GPL works, we would be happy to help.  As mentioned above,  *
- * we also offer alternative license to integrate Nmap into proprietary    *
+ * o Reads or includes copyrighted data files, such as Nmap's nmap-os-db   *
+ * or nmap-service-probes.                                                 *
+ *                                                                         *
+ * o Is designed specifically to execute Covered Software and parse the    *
+ * results (as opposed to typical shell or execution-menu apps, which will *
+ * execute anything you tell them to).                                     *
+ *                                                                         *
+ * o Includes Covered Software in a proprietary executable installer.  The *
+ * installers produced by InstallShield are an example of this.  Including *
+ * Nmap with other software in compressed or archival form does not        *
+ * trigger this provision, provided appropriate open source decompression  *
+ * or de-archiving software is widely available for no charge.  For the    *
+ * purposes of this license, an installer is considered to include Covered *
+ * Software even if it actually retrieves a copy of Covered Software from  *
+ * another source during runtime (such as by downloading it from the       *
+ * Internet).                                                              *
+ *                                                                         *
+ * o Links (statically or dynamically) to a library which does any of the  *
+ * above.                                                                  *
+ *                                                                         *
+ * o Executes a helper program, module, or script to do any of the above.  *
+ *                                                                         *
+ * This list is not exclusive, but is meant to clarify our interpretation  *
+ * of derived works with some common examples.  Other people may interpret *
+ * the plain GPL differently, so we consider this a special exception to   *
+ * the GPL that we apply to Covered Software.  Works which meet any of     *
+ * these conditions must conform to all of the terms of this license,      *
+ * particularly including the GPL Section 3 requirements of providing      *
+ * source code and allowing free redistribution of the work as a whole.    *
+ *                                                                         *
+ * As another special exception to the GPL terms, Insecure.Com LLC grants  *
+ * permission to link the code of this program with any version of the     *
+ * OpenSSL library which is distributed under a license identical to that  *
+ * listed in the included docs/licenses/OpenSSL.txt file, and distribute   *
+ * linked combinations including the two.                                  *
+ *                                                                         *
+ * Any redistribution of Covered Software, including any derived works,    *
+ * must obey and carry forward all of the terms of this license, including *
+ * obeying all GPL rules and restrictions.  For example, source code of    *
+ * the whole work must be provided and free redistribution must be         *
+ * allowed.  All GPL references to "this License", are to be treated as    *
+ * including the special and conditions of the license text as well.       *
+ *                                                                         *
+ * Because this license imposes special exceptions to the GPL, Covered     *
+ * Work may not be combined (even as part of a larger work) with plain GPL *
+ * software.  The terms, conditions, and exceptions of this license must   *
+ * be included as well.  This license is incompatible with some other open *
+ * source licenses as well.  In some cases we can relicense portions of    *
+ * Nmap or grant special permissions to use it in other open source        *
+ * software.  Please contact fyodor@nmap.org with any such requests.       *
+ * Similarly, we don't incorporate incompatible open source software into  *
+ * Covered Software without special permission from the copyright holders. *
+ *                                                                         *
+ * If you have any questions about the licensing restrictions on using     *
+ * Nmap in other works, are happy to help.  As mentioned above, we also    *
+ * offer alternative license to integrate Nmap into proprietary            *
  * applications and appliances.  These contracts have been sold to dozens  *
  * of software vendors, and generally include a perpetual license as well  *
  * as providing for priority support and updates.  They also fund the      *
  * continued development of Nmap.  Please email sales@insecure.com for     *
  * further information.                                                    *
  *                                                                         *
- * As a special exception to the GPL terms, Insecure.Com LLC grants        *
- * permission to link the code of this program with any version of the     *
- * OpenSSL library which is distributed under a license identical to that  *
- * listed in the included docs/licenses/OpenSSL.txt file, and distribute   *
- * linked combinations including the two. You must obey the GNU GPL in all *
- * respects for all of the code used other than OpenSSL.  If you modify    *
- * this file, you may extend this exception to your version of the file,   *
- * but you are not obligated to do so.                                     *
- *                                                                         *
  * If you received these files with a written license agreement or         *
  * contract stating terms other than the terms above, then that            *
  * alternative license agreement takes precedence over these comments.     *
@@ -66,8 +98,8 @@
  *                                                                         *
  * Source code also allows you to port Nmap to new platforms, fix bugs,    *
  * and add new features.  You are highly encouraged to send your changes   *
- * to nmap-dev@insecure.org for possible incorporation into the main       *
- * distribution.  By sending these changes to Fyodor or one of the         *
+ * to the dev@nmap.org mailing list for possible incorporation into the    *
+ * main distribution.  By sending these changes to Fyodor or one of the    *
  * Insecure.Org development mailing lists, or checking them into the Nmap  *
  * source code repository, it is understood (unless you specify otherwise) *
  * that you are offering the Nmap Project (Insecure.Com LLC) the           *
@@ -81,14 +113,13 @@
  *                                                                         *
  * This program is distributed in the hope that it will be useful, but     *
  * WITHOUT ANY WARRANTY; without even the implied warranty of              *
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU       *
- * General Public License v2.0 for more details at                         *
- * http://www.gnu.org/licenses/gpl-2.0.html , or in the COPYING file       *
- * included with Nmap.                                                     *
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the Nmap      *
+ * license file for more details (it's in a COPYING file included with     *
+ * Nmap, and also available from https://svn.nmap.org/nmap/COPYING         *
  *                                                                         *
  ***************************************************************************/
 
-/* $Id: ncat_connect.c 30306 2012-11-29 03:19:52Z david $ */
+/* $Id: ncat_connect.c 31563 2013-07-28 22:08:48Z fyodor $ */
 
 #include "base64.h"
 #include "nsock.h"
@@ -103,7 +134,6 @@
 #include <unistd.h>
 #include <netdb.h>
 #endif
-#include <assert.h>
 #include <stdlib.h>
 #include <string.h>
 #include <stdio.h>
@@ -157,6 +187,7 @@ static int verify_callback(int ok, X509_STORE_CTX *store)
        level. */
     if ((!ok && o.verbose) || o.debug > 1) {
         char digest_buf[SHA1_STRING_LENGTH + 1];
+        char *fp;
 
         loguser("Subject: ");
         X509_NAME_print_ex_fp(stderr, X509_get_subject_name(cert), 0, XN_FLAG_COMPAT);
@@ -165,7 +196,8 @@ static int verify_callback(int ok, X509_STORE_CTX *store)
         X509_NAME_print_ex_fp(stderr, X509_get_issuer_name(cert), 0, XN_FLAG_COMPAT);
         loguser_noprefix("\n");
 
-        assert(ssl_cert_fp_str_sha1(cert, digest_buf, sizeof(digest_buf)) != NULL);
+        fp = ssl_cert_fp_str_sha1(cert, digest_buf, sizeof(digest_buf));
+        ncat_assert(fp == digest_buf);
         loguser("SHA-1 fingerprint: %s\n", digest_buf);
     }
 
@@ -224,11 +256,12 @@ static void connect_report(nsock_iod nsi)
             X509 *cert;
             X509_NAME *subject;
             char digest_buf[SHA1_STRING_LENGTH + 1];
+            char *fp;
 
             loguser("SSL connection to %s:%hu.", inet_socktop(&peer), nsi_peerport(nsi));
 
             cert = SSL_get_peer_certificate((SSL *) nsi_getssl(nsi));
-            assert(cert != NULL);
+            ncat_assert(cert != NULL);
 
             subject = X509_get_subject_name(cert);
             if (subject != NULL) {
@@ -242,7 +275,8 @@ static void connect_report(nsock_iod nsi)
 
             loguser_noprefix("\n");
 
-            assert(ssl_cert_fp_str_sha1(cert, digest_buf, sizeof(digest_buf)) != NULL);
+            fp = ssl_cert_fp_str_sha1(cert, digest_buf, sizeof(digest_buf));
+            ncat_assert(fp == digest_buf);
             loguser("SHA-1 fingerprint: %s\n", digest_buf);
         } else {
 #if HAVE_SYS_UN_H
@@ -275,7 +309,7 @@ static const char *sock_to_url(const union sockaddr_u *su)
     if (su->storage.ss_family == AF_INET)
         Snprintf(buf, sizeof(buf), "%s:%hu", host_str, port);
     else if (su->storage.ss_family == AF_INET6)
-        Snprintf(buf, sizeof(buf), "[%s]:%hu]", host_str, port);
+        Snprintf(buf, sizeof(buf), "[%s]:%hu", host_str, port);
     else
         bye("Unknown address family in sock_to_url_host.");
 
@@ -493,9 +527,14 @@ int ncat_connect(void)
     if ((mypool = nsp_new(NULL)) == NULL)
         bye("Failed to create nsock_pool.");
 
-    if (o.debug > 1)
-        /* A trace level of 1 still gives you an awful lot. */
-        nsp_settrace(mypool, stderr, 1, nsock_gettimeofday());
+    if (o.debug >= 6)
+        nsock_set_loglevel(mypool, NSOCK_LOG_DBG_ALL);
+    else if (o.debug >= 3)
+        nsock_set_loglevel(mypool, NSOCK_LOG_DBG);
+    else if (o.debug >= 1)
+        nsock_set_loglevel(mypool, NSOCK_LOG_INFO);
+    else
+        nsock_set_loglevel(mypool, NSOCK_LOG_ERROR);
 
     /* Allow connections to broadcast addresses. */
     nsp_setbroadcast(mypool, 1);
@@ -516,7 +555,7 @@ int ncat_connect(void)
 
 #if HAVE_SYS_UN_H
         /* For DGRAM UNIX socket we have to use source socket */
-        if (o.af == AF_UNIX && o.udp)
+        if (o.af == AF_UNIX && o.proto == IPPROTO_UDP)
         {
             if (srcaddr.storage.ss_family != AF_UNIX) {
                 char *tmp_name = NULL;
@@ -552,7 +591,7 @@ int ncat_connect(void)
 
 #if HAVE_SYS_UN_H
         if (o.af == AF_UNIX) {
-            if (o.udp) {
+            if (o.proto == IPPROTO_UDP) {
                 nsock_connect_unixsock_datagram(mypool, cs.sock_nsi, connect_handler, NULL,
                                                 &targetss.sockaddr,
                                                 SUN_LEN((struct sockaddr_un *)&targetss.sockaddr));
@@ -563,13 +602,13 @@ int ncat_connect(void)
             }
         } else
 #endif
-        if (o.udp) {
+        if (o.proto == IPPROTO_UDP) {
             nsock_connect_udp(mypool, cs.sock_nsi, connect_handler,
                               NULL, &targetss.sockaddr, targetsslen,
                               inet_port(&targetss));
         }
 #ifdef HAVE_OPENSSL
-        else if (o.sctp && o.ssl) {
+        else if (o.proto == IPPROTO_SCTP && o.ssl) {
             nsock_connect_ssl(mypool, cs.sock_nsi, connect_handler,
                               o.conntimeout, NULL,
                               &targetss.sockaddr, targetsslen,
@@ -577,7 +616,7 @@ int ncat_connect(void)
                               NULL);
         }
 #endif
-        else if (o.sctp) {
+        else if (o.proto == IPPROTO_SCTP) {
             nsock_connect_sctp(mypool, cs.sock_nsi, connect_handler,
                               o.conntimeout, NULL,
                               &targetss.sockaddr, targetsslen,
@@ -685,7 +724,7 @@ int ncat_connect(void)
     }
 
 #if HAVE_SYS_UN_H
-    if (o.af == AF_UNIX && o.udp) {
+    if (o.af == AF_UNIX && o.proto == IPPROTO_UDP) {
         if (o.verbose)
             loguser("Deleting source DGRAM Unix domain socket. [%s]\n", srcaddr.un.sun_path);
         unlink(srcaddr.un.sun_path);
@@ -702,7 +741,7 @@ static void connect_handler(nsock_pool nsp, nsock_event evt, void *data)
     enum nse_status status = nse_status(evt);
     enum nse_type type = nse_type(evt);
 
-    assert(type == NSE_TYPE_CONNECT || type == NSE_TYPE_CONNECT_SSL);
+    ncat_assert(type == NSE_TYPE_CONNECT || type == NSE_TYPE_CONNECT_SSL);
 
     if (status == NSE_STATUS_ERROR) {
         loguser("%s.\n", socket_strerror(nse_errorcode(evt)));
@@ -711,7 +750,7 @@ static void connect_handler(nsock_pool nsp, nsock_event evt, void *data)
         loguser("%s.\n", socket_strerror(ETIMEDOUT));
         exit(1);
     } else {
-        assert(status == NSE_STATUS_SUCCESS);
+        ncat_assert(status == NSE_STATUS_SUCCESS);
     }
 
 #ifdef HAVE_OPENSSL
@@ -775,15 +814,13 @@ static void read_stdin_handler(nsock_pool nsp, nsock_event evt, void *data)
     char *buf, *tmp = NULL;
     int nbytes;
 
-    assert(type == NSE_TYPE_READ);
+    ncat_assert(type == NSE_TYPE_READ);
 
     if (status == NSE_STATUS_EOF) {
-        if (o.sendonly) {
-            /* In --send-only mode, exit after EOF on stdin. */
+        shutdown(nsi_getsd(cs.sock_nsi), SHUT_WR);
+        /* In --send-only mode or non-TCP mode, exit after EOF on stdin. */
+        if (o.proto != IPPROTO_TCP || (o.proto == IPPROTO_TCP && o.sendonly))
             nsock_loop_quit(nsp);
-        } else {
-            shutdown(nsi_getsd(cs.sock_nsi), SHUT_WR);
-        }
         return;
     } else if (status == NSE_STATUS_ERROR) {
         loguser("%s.\n", socket_strerror(nse_errorcode(evt)));
@@ -794,7 +831,7 @@ static void read_stdin_handler(nsock_pool nsp, nsock_event evt, void *data)
     } else if (status == NSE_STATUS_CANCELLED || status == NSE_STATUS_KILL) {
         return;
     } else {
-        assert(status == NSE_STATUS_SUCCESS);
+        ncat_assert(status == NSE_STATUS_SUCCESS);
     }
 
     buf = nse_readbuf(evt, &nbytes);
@@ -824,10 +861,13 @@ static void read_socket_handler(nsock_pool nsp, nsock_event evt, void *data)
     char *buf;
     int nbytes;
 
-    assert(type == NSE_TYPE_READ);
+    ncat_assert(type == NSE_TYPE_READ);
 
     if (status == NSE_STATUS_EOF) {
-        nsock_loop_quit(nsp);
+        Close(STDOUT_FILENO);
+        /* In --recv-only mode or non-TCP mode, exit after EOF on the socket. */
+        if (o.proto != IPPROTO_TCP || (o.proto == IPPROTO_TCP && o.recvonly))
+            nsock_loop_quit(nsp);
         return;
     } else if (status == NSE_STATUS_ERROR) {
         loguser("%s.\n", socket_strerror(nse_errorcode(evt)));
@@ -838,7 +878,7 @@ static void read_socket_handler(nsock_pool nsp, nsock_event evt, void *data)
     } else if (status == NSE_STATUS_CANCELLED || status == NSE_STATUS_KILL) {
         return;
     } else {
-        assert(status == NSE_STATUS_SUCCESS);
+        ncat_assert(status == NSE_STATUS_SUCCESS);
     }
 
     buf = nse_readbuf(evt, &nbytes);
@@ -863,7 +903,7 @@ static void write_socket_handler(nsock_pool nsp, nsock_event evt, void *data)
     enum nse_status status = nse_status(evt);
     enum nse_type type = nse_type(evt);
 
-    assert(type == NSE_TYPE_WRITE);
+    ncat_assert(type == NSE_TYPE_WRITE);
 
     if (status == NSE_STATUS_ERROR) {
         loguser("%s.\n", socket_strerror(nse_errorcode(evt)));
@@ -874,7 +914,7 @@ static void write_socket_handler(nsock_pool nsp, nsock_event evt, void *data)
     } else if (status == NSE_STATUS_CANCELLED || status == NSE_STATUS_KILL) {
         return;
     } else {
-        assert(status == NSE_STATUS_SUCCESS);
+        ncat_assert(status == NSE_STATUS_SUCCESS);
     }
 
     /* The write to the socket was successful. Allow reading more from stdin
@@ -887,12 +927,12 @@ static void idle_timer_handler(nsock_pool nsp, nsock_event evt, void *data)
     enum nse_status status = nse_status(evt);
     enum nse_type type = nse_type(evt);
 
-    assert(type == NSE_TYPE_TIMER);
+    ncat_assert(type == NSE_TYPE_TIMER);
 
     if (status == NSE_STATUS_CANCELLED || status == NSE_STATUS_KILL)
         return;
 
-    assert(status == NSE_STATUS_SUCCESS);
+    ncat_assert(status == NSE_STATUS_SUCCESS);
 
     loguser("Idle timeout expired (%d ms).\n", o.idletimeout);
 
