@@ -4,7 +4,7 @@
  *                                                                         *
  ***********************IMPORTANT NSOCK LICENSE TERMS***********************
  *                                                                         *
- * The nsock parallel socket event library is (C) 1999-2012 Insecure.Com   *
+ * The nsock parallel socket event library is (C) 1999-2013 Insecure.Com   *
  * LLC This library is free software; you may redistribute and/or          *
  * modify it under the terms of the GNU General Public License as          *
  * published by the Free Software Foundation; Version 2.  This guarantees  *
@@ -33,17 +33,18 @@
  *                                                                         *
  * Source code also allows you to port Nmap to new platforms, fix bugs,    *
  * and add new features.  You are highly encouraged to send your changes   *
- * to nmap-dev@insecure.org for possible incorporation into the main       *
- * distribution.  By sending these changes to Fyodor or one of the         *
- * Insecure.Org development mailing lists, it is assumed that you are      *
- * offering the Nmap Project (Insecure.Com LLC) the unlimited,             *
- * non-exclusive right to reuse, modify, and relicense the code.  Nmap     *
- * will always be available Open Source, but this is important because the *
- * inability to relicense code has caused devastating problems for other   *
- * Free Software projects (such as KDE and NASM).  We also occasionally    *
- * relicense the code to third parties as discussed above.  If you wish to *
- * specify special license conditions of your contributions, just say so   *
- * when you send them.                                                     *
+ * to the dev@nmap.org mailing list for possible incorporation into the    *
+ * main distribution.  By sending these changes to Fyodor or one of the    *
+ * Insecure.Org development mailing lists, or checking them into the Nmap  *
+ * source code repository, it is understood (unless you specify otherwise) *
+ * that you are offering the Nmap Project (Insecure.Com LLC) the           *
+ * unlimited, non-exclusive right to reuse, modify, and relicense the      *
+ * code.  Nmap will always be available Open Source, but this is important *
+ * because the inability to relicense code has caused devastating problems *
+ * for other Free Software projects (such as KDE and NASM).  We also       *
+ * occasionally relicense the code to third parties as discussed above.    *
+ * If you wish to specify special license conditions of your               *
+ * contributions, just say so when you send them.                          *
  *                                                                         *
  * This program is distributed in the hope that it will be useful, but     *
  * WITHOUT ANY WARRANTY; without even the implied warranty of              *
@@ -53,7 +54,7 @@
  *                                                                         *
  ***************************************************************************/
 
-/* $Id: nsock.h 30231 2012-11-12 20:44:37Z david $ */
+/* $Id: nsock.h 31563 2013-07-28 22:08:48Z fyodor $ */
 
 #ifndef NSOCK_H
 #define NSOCK_H
@@ -74,6 +75,8 @@
 #include <netinet/in.h>
 #include <arpa/inet.h>
 #include <sys/time.h>
+#else
+#include <winsock2.h>   /* for struct timeval... */
 #endif
 
 #if HAVE_SYS_UN_H
@@ -81,8 +84,8 @@
 
 #ifndef SUN_LEN
 #include <string.h>
-# define SUN_LEN(ptr) (sizeof(*(ptr)) - sizeof((ptr)->sun_path))     \
-                      + strlen ((ptr)->sun_path))
+#define SUN_LEN(ptr) ((sizeof(*(ptr)) - sizeof((ptr)->sun_path))    \
+                      + strlen((ptr)->sun_path))
 #endif
 #endif  /* HAVE_SYS_UN_H */
 
@@ -128,6 +131,45 @@ typedef void *nsock_ssl_session;
 typedef void *nsock_ssl_ctx;
 typedef void *nsock_ssl;
 
+typedef void *nsock_proxychain;
+
+
+/* Logging-related data structures */
+
+typedef enum {
+  /* --
+   * Actual message priority values */
+  NSOCK_LOG_DBG_ALL,
+  NSOCK_LOG_DBG,
+  NSOCK_LOG_INFO,
+  NSOCK_LOG_ERROR,
+  /* --
+   * No messages are issued by nsock with this value.
+   * Users can therefore set loglevel to NSOCK_LOG_NONE
+   * to disable logging */
+  NSOCK_LOG_NONE
+} nsock_loglevel_t;
+
+struct nsock_log_rec {
+  /* Message emission time */
+  struct timeval time;
+  /* Message log level */
+  nsock_loglevel_t level;
+  /* Source file */
+  const char *file;
+  /* Statement line in nsock source */
+  int line;
+  /* Function that emitted the message */
+  const char *func;
+  /* Actual log message */
+  char *msg;
+};
+
+/* Nsock logging function. This function receives all nsock log records whose
+ * level is greater than or equal to nsp loglevel. The rec structure is
+ * allocated and freed by nsock. */
+typedef void (*nsock_logger_t)(nsock_pool nsp, const struct nsock_log_rec *rec);
+
 
 /* ------------------- PROTOTYPES ------------------- */
 
@@ -147,7 +189,6 @@ enum nsock_loopstatus {
   NSOCK_LOOP_QUIT
 };
 
-
 enum nsock_loopstatus nsock_loop(nsock_pool nsp, int msec_timeout);
 
 /* Calling this function will cause nsock_loop to quit on its next iteration
@@ -179,15 +220,6 @@ void nsp_setud(nsock_pool nsp, void *data);
  * retrieve that data ... */
 void *nsp_getud(nsock_pool nsp);
 
-/* Sets a trace/debug level and stream.  A level of 0 (the default) turns
- * tracing off, while higher numbers are more verbose.  If the stream given is
- * NULL, it defaults to stdout.  This is generally only used for debugging
- * purposes. A level of 1 or 2 is usually sufficient, but 10 will ensure you get
- * everything.  The basetime can be NULL to print trace lines with the current
- * time, otherwise the difference between the current time and basetime will be
- * used (the time program execution starts would be a good candidate) */
-void nsp_settrace(nsock_pool nsp, FILE *file, int level, const struct timeval *basetime);
-
 /* Turns on or off broadcast support on new sockets. Default is off (0, false)
  * set in nsp_new(). Any non-zero (true) value sets SO_BROADCAST on all new
  * sockets (value of optval will be used directly in the setsockopt() call). */
@@ -231,6 +263,29 @@ nsock_pool nsp_new(void *userdata);
  * all outstanding iods are deleted. */
 void nsp_delete(nsock_pool nsp);
 
+/* Logging subsystem: set custom logging function.
+ * (See nsock_logger_t type definition). */
+void nsock_set_log_function(nsock_pool nsp, nsock_logger_t logger);
+
+nsock_loglevel_t nsock_get_loglevel(nsock_pool nsp);
+void nsock_set_loglevel(nsock_pool nsp, nsock_loglevel_t loglevel);
+
+/* Parse a proxy chain description string and build a nsock_proxychain object
+ * accordingly. If the optional nsock_pool parameter is passed in, it gets
+ * associated to the chain object. The alternative is to pass nsp=NULL and call
+ * nsp_set_proxychain() manually. Whatever is done, the chain object has to be
+ * deleted by the caller, using proxychain_delete(). */
+int nsock_proxychain_new(const char *proxystr, nsock_proxychain *chain, nsock_pool nspool);
+
+/* If nsock_proxychain_new() returned success, caller has to free the chain
+ * object using this function. */
+void nsock_proxychain_delete(nsock_proxychain chain);
+
+/* Assign a previously created proxychain object to a nsock pool. After this,
+ * new connections requests will be issued through the chain of proxies (if
+ * possible). */
+int nsp_set_proxychain(nsock_pool nspool, nsock_proxychain chain);
+
 /* nsock_event handles a single event.  Its ID is generally returned when the
  * event is created, and the event itself is included in callbacks
  *
@@ -267,8 +322,9 @@ enum nse_status {
                            nspool is being deleted -- you should free up any
                            resources you have allocated and exit.  Don't you
                            dare make any more async nsock calls!  */
-  NSE_STATUS_EOF        /* We got EOF and NO DATA -- if we got data first,
+  NSE_STATUS_EOF,       /* We got EOF and NO DATA -- if we got data first,
                            SUCCESS is reported (see nse_eof()). */
+  NSE_STATUS_PROXYERROR
 };
 
 enum nse_status nse_status(nsock_event nse);
@@ -459,6 +515,10 @@ nsock_event_id nsock_connect_unixsock_datagram(nsock_pool nsp, nsock_iod nsiod,
 nsock_event_id nsock_connect_tcp(nsock_pool nsp, nsock_iod nsiod, nsock_ev_handler handler, int timeout_msecs,
                                  void *userdata, struct sockaddr *ss, size_t sslen, unsigned short port);
 
+nsock_event_id nsock_connect_tcp_direct(nsock_pool nsp, nsock_iod nsiod, nsock_ev_handler handler,
+                                        int timeout_msecs, void *userdata, struct sockaddr *ss,
+                                        size_t sslen, unsigned short port);
+
 /* Request an SCTP association to another system (by IP address). The in_addr is
  * normal network byte order, but the port number should be given in HOST BYTE
  * ORDER.  ss should be a sockaddr_storage, sockaddr_in6, or sockaddr_in as
