@@ -7,7 +7,7 @@
  *                                                                         *
  ***********************IMPORTANT NSOCK LICENSE TERMS***********************
  *                                                                         *
- * The nsock parallel socket event library is (C) 1999-2012 Insecure.Com   *
+ * The nsock parallel socket event library is (C) 1999-2013 Insecure.Com   *
  * LLC This library is free software; you may redistribute and/or          *
  * modify it under the terms of the GNU General Public License as          *
  * published by the Free Software Foundation; Version 2.  This guarantees  *
@@ -36,17 +36,18 @@
  *                                                                         *
  * Source code also allows you to port Nmap to new platforms, fix bugs,    *
  * and add new features.  You are highly encouraged to send your changes   *
- * to nmap-dev@insecure.org for possible incorporation into the main       *
- * distribution.  By sending these changes to Fyodor or one of the         *
- * Insecure.Org development mailing lists, it is assumed that you are      *
- * offering the Nmap Project (Insecure.Com LLC) the unlimited,             *
- * non-exclusive right to reuse, modify, and relicense the code.  Nmap     *
- * will always be available Open Source, but this is important because the *
- * inability to relicense code has caused devastating problems for other   *
- * Free Software projects (such as KDE and NASM).  We also occasionally    *
- * relicense the code to third parties as discussed above.  If you wish to *
- * specify special license conditions of your contributions, just say so   *
- * when you send them.                                                     *
+ * to the dev@nmap.org mailing list for possible incorporation into the    *
+ * main distribution.  By sending these changes to Fyodor or one of the    *
+ * Insecure.Org development mailing lists, or checking them into the Nmap  *
+ * source code repository, it is understood (unless you specify otherwise) *
+ * that you are offering the Nmap Project (Insecure.Com LLC) the           *
+ * unlimited, non-exclusive right to reuse, modify, and relicense the      *
+ * code.  Nmap will always be available Open Source, but this is important *
+ * because the inability to relicense code has caused devastating problems *
+ * for other Free Software projects (such as KDE and NASM).  We also       *
+ * occasionally relicense the code to third parties as discussed above.    *
+ * If you wish to specify special license conditions of your               *
+ * contributions, just say so when you send them.                          *
  *                                                                         *
  * This program is distributed in the hope that it will be useful, but     *
  * WITHOUT ANY WARRANTY; without even the implied warranty of              *
@@ -56,9 +57,10 @@
  *                                                                         *
  ***************************************************************************/
 
-/* $Id: nsock_event.c 29414 2012-07-30 19:54:11Z henri $ */
+/* $Id: nsock_event.c 31563 2013-07-28 22:08:48Z fyodor $ */
 
 #include "nsock_internal.h"
+#include "nsock_log.h"
 #include "gh_list.h"
 
 #if HAVE_PCAP
@@ -113,8 +115,8 @@ char *nse_readbuf(nsock_event nse, int *nbytes) {
   msevent *me = (msevent *)nse;
 
   if (nbytes)
-    *nbytes = FILESPACE_LENGTH(&(me->iobuf));
-  return FILESPACE_STR(&(me->iobuf));
+    *nbytes = fs_length(&(me->iobuf));
+  return fs_str(&(me->iobuf));
 }
 
 static void first_ev_next(msevent *nse, gh_list_elem **first) {
@@ -139,7 +141,7 @@ static void first_ev_next(msevent *nse, gh_list_elem **first) {
 }
 
 void update_first_events(msevent *nse) {
-  switch(get_event_id_type(nse->id)) {
+  switch (get_event_id_type(nse->id)) {
     case NSE_TYPE_CONNECT:
     case NSE_TYPE_CONNECT_SSL:
       first_ev_next(nse, &nse->iod->first_connect);
@@ -176,7 +178,7 @@ void update_first_events(msevent *nse) {
  * it is the one which created it), in which case 0 can be passed to skip the
  * step.  This function returns zero if the event is not found, nonzero
  * otherwise. */
-int nsock_event_cancel(nsock_pool ms_pool, nsock_event_id id, int notify ) {
+int nsock_event_cancel(nsock_pool ms_pool, nsock_event_id id, int notify) {
   mspool *nsp = (mspool *)ms_pool;
   enum nse_type type;
   gh_list *event_list = NULL, *event_list2 = NULL;
@@ -186,12 +188,10 @@ int nsock_event_cancel(nsock_pool ms_pool, nsock_event_id id, int notify ) {
   assert(nsp);
 
   type = get_event_id_type(id);
-  if (nsp->tracelevel > 0) {
-    nsock_trace(nsp, "Event #%li (type %s) cancelled", id, nse_type2str(type));
-  }
+  nsock_log_info(nsp, "Event #%li (type %s) cancelled", id, nse_type2str(type));
 
   /* First we figure out what list it is in */
-  switch(type) {
+  switch (type) {
     case NSE_TYPE_CONNECT:
       event_list = &nsp->connect_events;
       break;
@@ -257,12 +257,11 @@ int msevent_cancel(mspool *nsp, msevent *nse, gh_list *event_list, gh_list_elem
     return 0;
   }
 
-  if (nsp->tracelevel > 0)
-    nsock_trace(nsp, "msevent_cancel on event #%li (type %s)", nse->id, nse_type2str(nse->type));
+  nsock_log_info(nsp, "msevent_cancel on event #%li (type %s)", nse->id, nse_type2str(nse->type));
 
   /* Now that we found the event... we go through the motions of cleanly
    * cancelling it */
-  switch(nse->type) {
+  switch (nse->type) {
     case NSE_TYPE_CONNECT:
     case NSE_TYPE_CONNECT_SSL:
       handle_connect_result(nsp, nse, NSE_STATUS_CANCELLED);
@@ -294,16 +293,14 @@ int msevent_cancel(mspool *nsp, msevent *nse, gh_list *event_list, gh_list_elem
   update_first_events(nse);
   gh_list_remove_elem(event_list, elem);
 
-  if (nsp->tracelevel > 8)
-    nsock_trace(nsp, "NSE #%lu: Removing event from list", nse->id);
+  nsock_log_debug_all(nsp, "NSE #%lu: Removing event from list", nse->id);
 
 #if HAVE_PCAP
 #if PCAP_BSD_SELECT_HACK
   if (nse->type == NSE_TYPE_PCAP_READ) {
-    if (nsp->tracelevel > 8)
-      nsock_trace(nsp, "PCAP NSE #%lu: CANCEL TEST pcap=%p read=%p curr=%p sd=%i",
-                  nse->id, &nsp->pcap_read_events, &nsp->read_events,
-                  event_list,((mspcap *)nse->iod->pcap)->pcap_desc);
+    nsock_log_debug_all(nsp, "PCAP NSE #%lu: CANCEL TEST pcap=%p read=%p curr=%p sd=%i",
+                        nse->id, &nsp->pcap_read_events, &nsp->read_events,
+                        event_list,((mspcap *)nse->iod->pcap)->pcap_desc);
 
     /* If event occurred, and we're in BSD_HACK mode, then this event was added to
      * two queues. read_event and pcap_read_event Of course we should
@@ -313,8 +310,7 @@ int msevent_cancel(mspool *nsp, msevent *nse, gh_list *event_list, gh_list_elem
       /* event is done, list is read_events and we're in BSD_HACK mode. So unlink
        * event from pcap_read_events */
       gh_list_remove(&nsp->pcap_read_events, nse);
-      if (nsp->tracelevel > 8)
-        nsock_trace(nsp, "PCAP NSE #%lu: Removing event from PCAP_READ_EVENTS", nse->id);
+      nsock_log_debug_all(nsp, "PCAP NSE #%lu: Removing event from PCAP_READ_EVENTS", nse->id);
     }
 
     if (((mspcap *)nse->iod->pcap)->pcap_desc >= 0 && event_list == &nsp->pcap_read_events) {
@@ -322,8 +318,7 @@ int msevent_cancel(mspool *nsp, msevent *nse, gh_list *event_list, gh_list_elem
        * So unlink event from read_events */
       gh_list_remove(&nsp->read_events, nse);
 
-      if (nsp->tracelevel > 8)
-        nsock_trace(nsp, "PCAP NSE #%lu: Removing event from READ_EVENTS", nse->id);
+      nsock_log_debug_all(nsp, "PCAP NSE #%lu: Removing event from READ_EVENTS", nse->id);
     }
   }
 #endif
@@ -376,8 +371,8 @@ nsock_event_id get_new_event_id(mspool *ms, enum nse_type type) {
   assert(type < NSE_TYPE_MAX);
 
   shiftbits = sizeof(nsock_event_id) * 8 - TYPE_CODE_NUM_BITS;
-  max_serial_allowed = ( 1 << shiftbits ) - 1;
-  if (serial == max_serial_allowed ) {
+  max_serial_allowed = (1 << shiftbits) - 1;
+  if (serial == max_serial_allowed) {
     /* then the next serial will be one because 0 is forbidden */
     ms->next_event_serial = 1;
   }
@@ -443,14 +438,12 @@ msevent *msevent_new(mspool *nsp, enum nse_type type, msiod *msiod, int timeout_
   nse->iod = msiod;
   nse->handler = handler;
   nse->userdata = userdata;
-  nse->time_created = nsock_tod;
 
-  if (nsp->tracelevel > 3) {
-    if (nse->iod == NULL)
-      nsock_trace(nsp, "msevent_new (IOD #NULL) (EID #%li)", nse->id);
-    else
-      nsock_trace(nsp, "msevent_new (IOD #%li) (EID #%li)", nse->iod->id, nse->id);
-  }
+  if (nse->iod == NULL)
+    nsock_log_debug(nsp, "msevent_new (IOD #NULL) (EID #%li)", nse->id);
+  else
+    nsock_log_debug(nsp, "msevent_new (IOD #%li) (EID #%li)", nse->iod->id,
+                    nse->id);
   return nse;
 }
 
@@ -460,12 +453,10 @@ msevent *msevent_new(mspool *nsp, enum nse_type type, msiod *msiod, int timeout_
  * remember to do this if you call msevent_delete() directly */
 void msevent_delete(mspool *nsp, msevent *nse) {
 
-  if (nsp->tracelevel > 3) {
-    if (nse->iod == NULL)
-      nsock_trace(nsp, "msevent_delete (IOD #NULL) (EID #%li)", nse->id);
-    else
-      nsock_trace(nsp, "msevent_delete (IOD #%li) (EID #%li)", nse->iod->id, nse->id);
-  }
+  if (nse->iod == NULL)
+    nsock_log_debug(nsp, "msevent_delete (IOD #NULL) (EID #%li)", nse->id);
+  else
+    nsock_log_debug(nsp, "msevent_delete (IOD #%li) (EID #%li)", nse->iod->id, nse->id);
 
   /* First free the IOBuf inside it if neccessary */
   if (nse->type == NSE_TYPE_READ || nse->type ==  NSE_TYPE_WRITE) {
@@ -474,8 +465,7 @@ void msevent_delete(mspool *nsp, msevent *nse) {
   #if HAVE_PCAP
   if (nse->type == NSE_TYPE_PCAP_READ) {
     fs_free(&nse->iobuf);
-    if (nsp->tracelevel > 5)
-      nsock_trace(nsp, "PCAP removed %lu",nse->id);
+    nsock_log_debug_all(nsp, "PCAP removed %lu", nse->id);
   }
   #endif
 
@@ -487,7 +477,7 @@ void msevent_delete(mspool *nsp, msevent *nse) {
 /* Takes an nse_type (as returned by nse_type() and returns a static string name
  * that you can use for printing, etc. */
 const char *nse_type2str(enum nse_type type) {
-  switch(type) {
+  switch (type) {
     case NSE_TYPE_CONNECT: return "CONNECT";
     case NSE_TYPE_CONNECT_SSL: return "SSL-CONNECT";
     case NSE_TYPE_READ: return "READ";
@@ -502,7 +492,7 @@ const char *nse_type2str(enum nse_type type) {
 /* Takes an nse_status (as returned by nse_status() and returns a static string
  * name that you can use for printing, etc. */
 const char *nse_status2str(enum nse_status status) {
-  switch(status) {
+  switch (status) {
     case NSE_STATUS_NONE: return "NONE";
     case NSE_STATUS_SUCCESS: return "SUCCESS";
     case NSE_STATUS_ERROR: return "ERROR";
@@ -510,8 +500,15 @@ const char *nse_status2str(enum nse_status status) {
     case NSE_STATUS_CANCELLED: return "CANCELLED";
     case NSE_STATUS_KILL: return "KILL";
     case NSE_STATUS_EOF: return "EOF";
+    case NSE_STATUS_PROXYERROR: return "PROXY ERROR";
     default:
       return "UNKNOWN!";
   }
 }
 
+int msevent_timedout(msevent *nse) {
+  if (nse->event_done)
+    return 0;
+
+  return (nse->timeout.tv_sec && !TIMEVAL_AFTER(nse->timeout, nsock_tod));
+}
